
namespace eval acknowledgement {}

proc ack_scheduler {} {
	return 1
}

ad_proc -private acknowledgement::affected_users { ack_id group_id } {
	Procedure is work in progress, currently hard coded returns users that
	would be affected by a ban operation 
} {
    if { [empty_string_p $group_id] } {
    	set sql_str {
    		select user_id from users
    		where not exists (select 1 from acknowledgment_responses where ack_id = :ack_id and user_id = users.user_id)
    	}
    } else {
    	set sql_str {
    		select member_id from group_member_map 
    		where group_id = $group_id
    		and not exists (select 1 from acknowledgment_responses where ack_id = :ack_id and user_id = member_id)
    	}
    }
    return [db_list get_user_list $sql_str]
}

ad_proc -public acknowledgement::lock_accounts { ack_id } {
	Lock accounts of users who have not accepted a term of use 
} {
	array set ack_info [get_data -ack_id $ack_id]
    foreach element [affected_users $ack_id $ack_info(group_id)] {
    	acs_user::change_state -user_id $element -state "banned"
    }
	return [llength $lock_list]
}

ad_proc -public acknowledgement::get_data { 
	{-shortname ""}
	{-ack_id ""}
} {
	Get information about a certain term of use
} {
	if { [empty_string_p $ack_id] } {
		if { [empty_string_p $shortname] } { return 0 }
		set sql_str "select ack_id from acknowledgments where shortname = :shortname"
		set ack_id [db_string get_data $sql_str -default 0]
		if { $ack_id == 0 } { return 0 }
	}
	set fields [list ack_id shortname title description global_p ack_mode ack_phrase group_id due_date]
	set db_result [db_0or1row get_data "select [join $fields ","] from acknowledgments where ack_id = :ack_id"]
	if { $db_result == 0 } { return 0 }
	set output [list]
	foreach element $fields {
		lappend output $element [set $element]
	}
	ns_log Notice "Returning $output"
	return $output
}

ad_proc -public acknowledgement::new { ack_id shortname title description global_p mode phrase group_id date } {
	Create a new acknowledgement item 
} {
	set sql_str "select acknowledgment__new(:ack_id,:shortname,:title,:description,:global_p,:mode,:phrase,:group_id,:date)"
	return [db_string write_data $sql_str]
}

ad_proc -public acknowledgement::edit { ack_id shortname title description global_p mode phrase group_id date } {
	Edit item data 
} {
	set sql_str "select acknowledgment__edit(:ack_id,:shortname,:title,:description,:global_p,:mode,:phrase,:group_id,:date)"
	return [db_string write_data $sql_str]
}

ad_proc -public acknowledgement::delete { ack_id } {
	Delete an existing acknowledgement item 
} {
	return [db_string write_data "select acknowledgment__delete(:ack_id)"]
}

ad_proc -public acknowledgement::repent_url { user_id } {
	The url that users are presented to accept an acknowledgement 
	during the login procedure 
} {
	return "<a href='/acknowledgements/repent?user_id=$user_id'>[_ acknowledgement.repent]</a>"
}

ad_proc -public acknowledgement::overdue_acks_p { user_id } {
	Get the number of acknowledgement items that are "overdue" for a certain user
} {
	set sql_str {
		select 1 from acknowledgments a 
		where global_p = 't' 
		and not exists (select 1 from acknowledgment_responses where ack_id = a.ack_id and user_id = :user_id)
		and ( due_date is null or due_date <= now()::date )
		and (a.group_id is null or ((select count(*) from group_member_map where group_id = a.group_id and member_id = :user_id) > 0))
	}
	return [llength [db_list get_data $sql_str]]
}

ad_proc -public acknowledgement::get_for_user { user_id {filter "overdue"} } {
	Get the acknowledgement items that are "overdue" for a certain user
	or if filter is "all" then return all items
} {
	set sql_str {
		select a.ack_id, a.title, a.due_date from acknowledgments a 
		where global_p = 't' 
		and not exists (select 1 from acknowledgment_responses where ack_id = a.ack_id and user_id = :user_id)
	}
	if { $filter == "overdue" } { append sql_str " and ( due_date is null or due_date >= now()::date )" }
	append sql_str {
		and (a.group_id is null or ((select count(*) from group_member_map where group_id = a.group_id and member_id = :user_id) > 0))
		order by a.ack_id
	}
	return [db_list_of_lists get_data $sql_str]
}

ad_proc -public acknowledgement::record_response { ack_id {user_id 0} } {
	Record the response of a user to a certain acknowledgement item
} {
	if { $user_id == 0 } { set user_id [ad_get_user_id] }
	db_string write_data "select acknowledgment__new_response(:ack_id,:user_id)"
    array set ack_info [get_data -ack_id $ack_id]
	ns_cache set ack_info_cache "${ack_info(shortname)}.$user_id" 1
	return 
}

ad_proc -public acknowledgement::reset_responses { ack_id } {
	Reset all responses that user have given to an acknowledement item so far
} {
	return [db_string write_data "select acknowledgment__reset(:ack_id)"]
}

proc acknowledgement::response_exists_p { ack_id user_id } {
	return [db_string get_data "select 1 from acknowledgment_responses where ack_id = :ack_id and user_id = :user_id" -default 0]
}

ad_proc -public acknowledgement::response_url {} {
	An url that users are redirected to when they have to accept an acknowledgement item
} {
	return "/acknowledgements/record_response"
}

ad_proc -public acknowledgement::require_response { shortname } {
	Programmatically check whether the current user has accepted a certain
	acknowledgement item. The procedure checks whether the user has to
	accept the acknowledgement item (because of group constraints) or not
	and records the result in the cache ack_info_cache. If the user has not 
	accepted the item he is redirected to the url returned from acknowledgement::response_url
	
	@param shortname The shortname of an acknowledement item
	@see acknowledgement::response_url
} {
    set user_id [ad_get_user_id]
	if { ![ns_cache get ack_info_cache "${shortname}.$user_id" result] } {
		set result [acknowledgement::require_response_no_cache $user_id $shortname]
		ns_cache set ack_info_cache "${shortname}.$user_id" $result
	}
	if { $result != 1 } {
		set return_url [ns_conn url]
		if { [ns_conn query] != "" } { append return_url "?[ns_conn query]" }
		ad_returnredirect "$result&return_url=$return_url"
		ad_script_abort
	}
	return 1
}

ad_proc -private acknowledgement::require_response_no_cache { user_id shortname } {
	Checks whether a certain user has to accept a certain acknowledgement item
	Called by acknowledgement::require_response
	
	@see acknowledgement::require_response
} {
    array set ack_info [get_data -shortname $shortname]
    set response_p [response_exists_p $ack_info(ack_id) $user_id]
    if { $response_p } { return 1 }
    if { [empty_string_p $ack_info(group_id)] } {
    	return "[response_url]?id=$ack_info(ack_id)"
    } elseif { [group::member_p -user_id $user_id -group_id $ack_info(group_id) -cascade] } {
    	return "[response_url]?id=$ack_info(ack_id)"
    }
    return 1
}

ns_log Notice "Loaded acknowledgement procs"
