ad_page_contract {
    Page for editing and adding acknowledgements.
    
    @author Peter Alberer(peter@alberer.com)
    @creation-date 2003-10-06
} {
    id:integer,optional
    {ad_form_mode display}
    {lock_count ""}
}

proc all_groups_list {} {
	#get a list of groups. when dotlrn is installed, use dotlrn communities instead
	if { [apm_package_installed_p dotlrn] } {
		set sql_str { 
			select community_id as group_id, pretty_name || ' (' || community_key || ')' as group_name
			from dotlrn_communities
			order by group_name
		}
	} else {
		set sql_str { 
			select group_id, group_name from groups order by group_name
		}
	}
	set output [list [list "No Group" ""]]
	db_foreach get_data $sql_str {
		lappend output [list $group_name $group_id] 
	}
	return $output
}

set page_title [_ acknowledgement.details]
set context [list $page_title]

set page_title ""
if { [exists_and_not_null id] } {
    # Initial request in display or edit mode or a submit of the form
    set ack_exists_p [db_string ack_exists_p {
        select count(*)
        from acknowledgments
        where ack_id = :id
    }]
} else {
    # Initial request in add mode
    set page_title [_ acknowledgement.new_ack]
    set ad_form_mode edit
    set ack_exists_p 0
}

set shortname_help 	[_ acknowledgement.shortname_help]
set global_help 	[_ acknowledgement.global_help]
set mode_help 		[_ acknowledgement.mode_help]
set phrase_help 	[_ acknowledgement.phrase_help]
set group_help 		[_ acknowledgement.group_help]
set due_date_help 	[_ acknowledgement.due_date_help]

set form_widgets {
    id:key(acknowledgment_seq)
    {shortname:text
  		{html {size 20}}
        {label "#acknowledgement.shortname#"}
        {help_text $shortname_help}
    }        
	{title:text
		{html {size 50}}
		{label "#acknowledgement.title#"}
	}
	{description:text(textarea)
		{html {cols 60 rows 13}}
		{label "#acknowledgement.description#"}
	}
	{global_p:text(radio)
        {label "#acknowledgement.global#"}
        {options {{Ja t} {Nein f}}}
		{help_text $global_help}
    }
	{ack_mode:text(select)
		{label "#acknowledgement.mode#"}
		{options {{Checkbox checkbox} {Phrase phrase} {Passwort password}}}
		{help_text $mode_help}
	}
	{ack_phrase:text,optional
		{label "#acknowledgement.phrase#"}
		{help_text $phrase_help}
	}
	{group_id:integer(select),optional
		{label "#acknowledgement.group#"}
		{help_text $group_help}
		{options {[all_groups_list]}}
	}
	{due_date:date,optional
		{label "#acknowledgement.due_date#"}
		{format "DD MONTH YYYY"}
		{help_text $due_date_help}
	}
}

set commands_html ""
set operations_message ""

ad_form -name acks -mode $ad_form_mode -form $form_widgets -cancel_url "." \
-edit_request {
	db_1row get_data "select * from acknowledgments where ack_id = :id"
	#hmm i do not really know how to get the form accept my date value from the database without the following code
	if { $due_date != "" } { 
		regexp {([0-9]+)-([0-9]+)-([0-9]+)} $due_date _ year month day
		set due_date "$year $month $day {} {} {} {DD MONTH YYYY}" 
	} 
	append commands_html "<li><a href='lock-accounts?id=$id'>[_ acknowledgement.lock_users_not_signed]</a></li>"
	set user_count [db_string get_data "select count(*) from acknowledgment_responses where ack_id = :id"]
	if { $user_count > 0 } {
		append commands_html "<li>[_ acknowledgement.signed_by_users]: <a href='responses?id=$id'>$user_count</a>. <a href='reset-responses?id=$id&return_url=[ad_conn url]?id=$id' onclick=\"return confirm('[_ acknowledgement.are_you_sure]');\">[_ acknowledgement.reset_responses]</a></li>"
	}
} -new_data {
	form get_values acks id shortname title description global_p ack_mode ack_phrase group_id due_date
	if { $due_date != "" } { set date "[lindex $due_date 0]-[lindex $due_date 1]-[lindex $due_date 2]" } else { set date "" }
	acknowledgement::new $id $shortname $title $description $global_p $ack_mode $ack_phrase $group_id $date
} -edit_data {
	form get_values acks id shortname title description global_p ack_mode ack_phrase group_id due_date
	if { $due_date != "" } { set date "[lindex $due_date 0]-[lindex $due_date 1]-[lindex $due_date 2]" } else { set date "" }
	acknowledgement::edit $id $shortname $title $description $global_p $ack_mode $ack_phrase $group_id $date
} -after_submit {
    ad_returnredirect [export_vars -base [ad_conn url] {id}]
}

if { $lock_count != "" } { set operations_message "<p>$lock_count [_ acknowledgement.accounts_been_locked]</p>" }
if { $commands_html != "" } { set commands_html "<ul>$commands_html</ul>" }
