# this file is meant to be included with the following parameters
#
# folder_id - if you are creating a new page
# revision_id - if you are editing a page revision
# return_url - requires a return_url, so after creating or editing a folder it redirect to this url
# form_mode - either "edit" or "display"

bcms::ui::base::check_include_vars

ad_form -name simpleform -mode $form_mode -form {
    revision_id:key

    {name:text(text) {help_text {no spaces, no special characters}} {label "Page Name"}}
    {title:text(text) {label "Title"}}
    {description:text(textarea),optional {html {rows 5 cols 80}} {label "Description"}}
    {content:text(textarea),optional {html {rows 20 cols 80}} {label "Page Content"}}
    {folder_id:integer(hidden),optional {value $folder_id}}
    {item_id:integer(hidden),optional {value $item_id}}
    {return_url:text(hidden) {value $return_url}}

} -validate {
    {name
        {![bcms::item::is_item_duplicate_p -url $name -root_id $folder_id -item_id $item_id]}
        "Page Name already exists, <br /> please use another Page Name"
    }
} -edit_request {

    array set one_revision [bcms::revision::get_revision -revision_id $revision_id]
    set item_id $one_revision(item_id)
    set name $one_revision(name)
    set title $one_revision(title)
    set content $one_revision(content)
    set description $one_revision(description)
    set folder_id $one_revision(parent_id)

} -edit_data {

    bcms::revision::set_revision -revision_id $revision_id \
       -title $title -content $content -description $description
    if {[info exists name]} {
        bcms::item::set_item -item_id $item_id -name $name
        # we have renamed the page, we need to redirect on the new name
        bcms::ui::base::redirect_after_rename -item_id $item_id
    }

} -new_data {

    # create the page and revision
    set item_id [bcms::create_page -page_name $name -folder_id $folder_id -mime_type "text/html" \
                     -title $title -description $description -page_body $content]
} -after_submit {

    ad_returnredirect $return_url
    ad_script_abort

}

ad_return_template "/packages/bcms-ui-base/resources/simple-form"


