ad_library {

    Procs for content repository keywords

    @author Jun Yamog
    @creation-date Aug 2003
    @cvs-id $Id: bcms-category-procs.tcl,v 1.8 2004/02/17 14:38:24 lars Exp $

    some are derived from Deds classified-ads:categories
}

namespace eval bcms::category {}

ad_proc -public bcms::category::create_category {
    {-heading:required}
    {-description ""}
    {-parent_id ""}
    {-creation_user_id}
    {-creation_ip}
} {

    creates a new keyword that's used for categorization

    @param heading        The heading of the new keyword
    @param description    The description of the new keyword
    @param parent_id      The parent of this new keyword if it is a subcategory

} {
    if {![info exists creation_user_id]} { set creation_user_id [ad_conn user_id] }
    if {![info exists creation_ip]} { set creation_ip [ad_conn peeraddr] }

    return [db_exec_plsql create_category {}]
}


ad_proc -public bcms::category::set_category {
   {-category_id:required}
   {-heading}
   {-description}
} {

    sets/edits properties of an existing category

    @param category_id    The keyword id of the category to edit
    @param heading        The new heading to use
    @param description    The new description to use

} {

    if {[info exists heading]} {
        db_dml set_heading {}
    }

    if {[info exists description]} {
        db_dml set_description {}
    }

}


ad_proc -public bcms::category::delete_category {
    {-category_id:required}
} {

    deletes a category

    @param category_id The category id of the category to delete

} {
    db_exec_plsql delete_category {}
}


ad_proc -public bcms::category::get_category {
   {-category_id:required}
} {

    returns the data for a particular category

    @param category_id The category id of the category to get the data

    @return an array with the following properties:
    category_id, parent_id, heading, description, has_children

} {

    if {[db_0or1row get_category {} -column_array category]} {
        return [array get category]
    } else {
        return {}
    }

}


ad_proc -public bcms::category::list_categories {
    {-parent_id}
    {-return_list:boolean}
    {-multirow_name bcms_list_category}
} {

    returns the categories under a parent

    @parent_id                  id of the category which will list the category under it, if not supplied
                                it will list all the root keywords
    @param multirow_name        you can supply your own multirow name
    @param return_list          if supplied a list of ns_sets otherwise it
                                returns a multirow uses the multirow_name if given

    @return makes a multirow of categories or list of ns sets with the following properties:
    category_id, parent_id, heading, description, has_children

} {

    if {[info exists parent_id]} {
        set where_str "parent_id = :parent_id"
    } else {
        set where_str "parent_id is null"
    }

    if {$return_list_p} {
        # return a list of lists since its not easy to pass multirow from proc to proc
        return [db_list_of_ns_sets get_categories {}]
    } else {
        # make a multirow, it is likely that this proc is not being called from another proc
        db_multirow $multirow_name get_categories {}
    }

}


ad_proc -public bcms::category::tree_categories {
    {-parent_id:required}
    {-return_list:boolean}
    {-multirow_name bcms_tree_category}
} {

    returns the all sub categories under the root id

    @parent_id                  category id of root node of the tree to return
    @param multirow_name        you can supply your own multirow name
    @param return_list          if supplied a list of ns_sets otherwise it
                                returns a multirow uses the multirow_name if given

    @return all sub categories under the root_id with the following attributes:
    category_id, parent_id, heading, description, has_children, path
} {

    if {$return_list_p} {
        # return a list of lists since its not easy to pass multirow from proc to proc
        return [db_list_of_ns_sets get_categories {}]
    } else {
        # make a multirow, it is likely that this proc is not being called from another proc
        db_multirow $multirow_name get_categories {}
    }

}


ad_proc -public bcms::category::get_category_path {
    {-category_id:required}
    {-root_id}
    {-prepend_path ''}
    {-multirow_name bcms_category_path}
    {-return_list:boolean}
} {
    returns a path from the root_id to the category_id

    @param category_id       build the path to this category
    @param root_id           build the path from this root, if not supplied it will use the root node of the
                             current category
    @param prepend_path      you can add a string to prepend the path column
    @param multirow_name     you can supply your own multirow name
    @param return_list       if supplied a list of ns_sets otherwise it
                             returns a multirow uses the multirow_name if given

    @return a multirow/list with the following elements:
    category_id, path, heading
} {

    if {![info exists root_id]} {
        # if the root_id does not exists then start from the root
        array set root [bcms::category::get_root -category_id $category_id]
        if {[array size root] > 0} {
            set root_id $root(category_id)
        } else {
            ns_log notice "bcms: category does not exists"
        }
    }

    if {$return_list_p} {
        return [db_list_of_ns_sets get_category_path {}]
    } else {
        db_multirow $multirow_name get_category_path {}
    }
}


ad_proc -public bcms::category::get_root {
    {-category_id:required}
} {
    gets the root id of a category tree, supplied with any category underneath it

    @param category_id    a node of the category tree

    @return an array with the following properties:
    category_id, parent_id, heading, description, has_children
} {

    if {[db_0or1row get_root {} -column_array category]} {
        return [array get category]
    } else {
        return {}
    }

}

