ad_library {

	Utility procs and procs that are not grouped/belong to the other library 

	@author Jun Yamog
	@creation-date 10 Aug 2002
	@cvs-id $Id: bcms-procs.tcl,v 1.10 2004/02/17 14:38:25 lars Exp $
}

namespace eval bcms {}

# for possible contribution
ad_proc -public bcms::get_table_name {
    -object_type:required
} {
    @param object_type     the object_type of an object, normally the content_type since we are dealing with cms

    @return the table name of a particular object_type
} {
    return [db_string get_table_name {}] 
}


# for possible contribution
ad_proc -public bcms::get_next_objectid { } {
    @return the next acs objectID sequence from the database
} {
    return [db_string get_next_objectid {}]
}


ad_proc -public bcms::create_page {
    {-page_name:required}
    {-folder_id:required}
    {-mime_type "text/plain"}
    {-title:required}
    {-description ""}
    {-page_body ""}
    {-creation_user}
    {-creation_ip}
    {-content_type content_revision}
} {
    creates a new page and its first revision

    @param page_name         name of the page which will be used for the url
    @param folder_id         the folder_id which the page will be under
    @param mime_type         mime_type of the page
    @param title             title of the page
    @param description       description of the page
    @param page_body         the contents of a page
    @param creation_user  user_id creating this page
    @param creation_ip  ip address which this page is created

    @return the page_id of the newly created page
} {
    if {![info exists creation_user]} { set creation_user [ad_conn user_id] }
    if {![info exists creation_ip]} { set creation_ip [ad_conn peeraddr] }

    db_transaction {
        set page_id [bcms::item::create_item -item_name $page_name -parent_id $folder_id -content_type $content_type \
                         -storage_type text -creation_user $creation_user -creation_ip $creation_ip]

        set revision_id [bcms::revision::add_revision -item_id $page_id \
                             -title $title -content $page_body -description $description \
                             -mime_type $mime_type \
                             -creation_user $creation_user -creation_ip $creation_ip]
    }

    return $page_id

}

ad_proc -public bcms::upload_file {
    {-file_name:required}
    {-folder_id:required}
    {-title:required}
    {-description ""}
    {-upload_file:required}
    {-creation_user}
    {-creation_ip}
    {-content_type content_revision}
} {
    uploads a new file and its first revision

    @param file_name         name of the file which will be used for the url
    @param folder_id         the folder_id which the file will be under
    @param mime_type         mime_type of the file
    @param title             title of the file
    @param description       description of the file
    @param upload_file       the file you want to upload
    @param creation_user  user_id creating this page
    @param creation_ip  ip address which this page is created

    @return the file_id of the newly created file
} {
# we need to check for an existing conn before doing this in tcl
# probably better to just set them to NULL

    if {![info exists creation_user]} { set creation_user [ad_conn user_id] }
    if {![info exists creation_ip]} { set creation_ip [ad_conn peeraddr] }

    db_transaction {
        set file_id [bcms::item::create_item -item_name $file_name -parent_id $folder_id -content_type $content_type \
                         -storage_type file -creation_user $creation_user -creation_ip $creation_ip]

        set revision_id [bcms::revision::upload_file_revision -item_id $file_id \
                             -title $title -description $description \
                             -upload_file $upload_file \
                             -creation_user $creation_user -creation_ip $creation_ip]

    }

    return $file_id

}

ad_proc -public bcms::parse_properties {
    {-properties:required}
    {-return_format:required}
    {-uplevel_level 1}
} {
    parses properties returns strings to be used for insert or update sql strings

    @param properites              A list of properties / columns to insert   format: { { property value } ... { property value } }
                                   {property:noquote value} will directly use what is in value. 
                                   Use [list [list prop1 $val1] [list $prop2 $val2]] to generate.

    @param return_format           either "insert_values", "insert_columns" or "update"
    @param uplevel_level           if you need to set the quoted vars to another level use this to set the proper uplevel  
} {

    # create the properties
    set properties_list {}
    set values_list {}
    foreach one_property $properties {
        set property [lindex $one_property 0]
        set value [lindex $one_property 1]
        if {[regexp {(.+):noquote$} $property match property]} {
            #if noquote then use what is in the value
            lappend values_list $value
        } else {
            lappend values_list :__$property
            upvar $uplevel_level "__$property" localvar
            set localvar $value
        }
        lappend properties_list $property
    }

    switch -exact $return_format {
        insert_values { 
            set insert_values "[join $values_list ","]"
            return $insert_values 
        }
        insert_columns { 
            set insert_columns "[join $properties_list ","]"
            return $insert_columns
        }
        update {
            set index 0
            set update_list {}
            foreach one_property $properties_list {
                lappend update_list "$one_property = [lindex $values_list $index]"
                incr index
            }
            set update_string "[join $update_list ", "]"
            return $update_string
        }
        default {
            error "invalid return_format"
        }
    }
}
