ad_library {

	Procs for Content Repository's content revision

	@author Jun Yamog
	@creation-date 06 Oct 2002
	@cvs-id $Id: bcms-revision-procs.tcl,v 1.20 2004/03/12 14:01:42 peterm Exp $

TODO: add back the lob support

}

namespace eval bcms::revision {}

# TODO merge with upload_file_revision
ad_proc -public bcms::revision::add_revision {
    {-item_id:required}
    {-title:required}
    {-content_type}
    {-mime_type "text/plain"}
    {-description ""}
    {-content ""}
    {-creation_user}
    {-creation_ip}
    {-additional_properties}
} {
    Adds a new revision, you pass additional properties/columns for your custom
    content type

    @param item_id                 item_id of this revision is adding to
    @param title                   title of the revision
    @param content_type            content_type of the item, if not passed it will determine the content_type
    @param mime_type               mime type of this revision
    @param description             description of the revision
    @param content                 content of the revision
    @param creation_user        user_id creating this item
    @param creation_ip        ip address which this item is created
    @param additional_properites   A list of properties / columns to insert   format: { { property value } ... { property value } }
                                   {property:noquote value} will directly use what is in value. 
                                   Use [list [list prop1 $val1] [list $prop2 $val2]] to generate.

    @return revision_id of newly created revision
} {
    if {![info exists creation_user]} { set creation_user [ad_conn user_id] }
    if {![info exists creation_ip]} { set creation_ip [ad_conn peeraddr] }

    if { [exists_and_not_null additional_properties] } {
        set addtnl_properties ", [bcms::parse_properties -properties $additional_properties -return_format insert_columns]"
        set addtnl_values ", [bcms::parse_properties -properties $additional_properties -return_format insert_values]"
    } else {
        set addtnl_properties ""
        set addtnl_values ""
    }

    if ![info exists content_type] {
        set content_type [bcms::item::get_content_type -item_id $item_id]
    }
    set table_name [bcms::get_table_name -object_type $content_type]

    db_transaction {
        set revision_id [bcms::get_next_objectid]
        db_dml insert_revision {}
    }

    ns_log notice "bcms::revision::add_revision created new revision $revision_id"
    return $revision_id

}

ad_proc -public bcms::revision::upload_file_revision {
    {-item_id:required}
    {-title:required}
    {-content_type}
    {-description ""}
    {-upload_file:required}
    {-mime_type}
    {-creation_user}
    {-creation_ip}
    {-additional_properties}
} {
    Adds a new revision to a content item stored in file system at
    acs_root/content-repository-content-files.

    @param item_id                 item_id of this revision is adding to
    @param title                   title of the revision
    @param content_type            content_type of the item, if not passed it will determine the content_type
    @param mime_type               mime type of this revision
    @param description             description of the revision
    @param upload_file             a 'file' form builder datatype
    @param creation_user        user_id creating this item
    @param creation_ip        ip address which this item is created
    @param additional_properites   a list of additional properties / columns to insert   format: {property value ... property value}
                                   {property:noquote value} will directly use what is value
} {
    if { ![info exists creation_user] } { set creation_user [ad_conn user_id] }
    if { ![info exists creation_ip] } { set creation_ip [ad_conn peeraddr] }

    if { ![info exists content_type] } {
        set content_type [bcms::item::get_content_type -item_id $item_id]
    }
    set table_name [bcms::get_table_name -object_type $content_type]

    # get the file properties
    set upload_filename [template::util::file::get_property filename $upload_file]
    set upload_tmpfile [template::util::file::get_property tmp_filename $upload_file]

    if { ![exists_and_not_null mime_type] } {
        set mime_type [template::util::file::get_property mime_type $upload_file]
    }
    if { ![exists_and_not_null mime_type] } {
        set mime_type [cr_filename_to_mime_type -create $upload_filename]
    }

    if { [exists_and_not_null additional_properties] } {
        set addtnl_properties ", [bcms::parse_properties -properties $additional_properties -return_format insert_columns]"
        set addtnl_values ", [bcms::parse_properties -properties $additional_properties -return_format insert_values]"
    } else {
        set addtnl_properties ""
        set addtnl_values ""
    }

    db_transaction {
        set revision_id [bcms::get_next_objectid]
        # create the file in the file system and get its name and size
        set tmp_filename [cr_create_content_file $item_id $revision_id $upload_tmpfile]
        set tmp_filesize [cr_file_size $tmp_filename]	

        # since we are using the file system we will set content to the filename in the fs
        set content $tmp_filename
        db_dml insert_revision {}
        db_dml update_file_size {}
    }

    return $revision_id

}

ad_proc -public bcms::revision::list_revisions {
    {-item_id:required}
    {-return_list:boolean false}
    {-multirow_name}
} {
    gets the revisions of a content item

    @param item_id              content item where to get the different revisions
    @param multirow_name        you can supply your own multirow name
    @param return_list          if supplied a list of ns_sets otherwise it
                                returns a multirow uses the multirow_name if given

    @return the revisions of a content item.  with the following data:
    revision_id, title, live_revision_id, creation_date, description and creation user's first_names and last_name
} {
    # TODO: content_type is passed not sure if if need to,
    # maybe can pass in additional_columns and then make use to return
    # other columns
    set content_type "content_revision"

    set table_name [bcms::get_table_name -object_type $content_type]
    if $return_list_p {
        return [db_list_of_ns_sets get_revisions {}]
    } else {
        if ![info exists multirow_name] { set multirow_name bcms_revision_list }
        db_multirow $multirow_name get_revisions {}
    }
}

ad_proc -public bcms::revision::get_revision {
    {-revision_id:required}
    {-additional_properties}
} {
    Get the revision info on a single revision in array-list format.

    @param revision_id            revision_id of the version you want to retrieve
    @param additional_properties  a list of property names that you add to the array returned

    @return an array that contains item_id, name, title, description,
    content, content_type, publish_status, latest_revision, live_revision, storage_type
    and whatever additional_properties is supplied
} {
    set addtnl_select ""
    set addtnl_from ""
    set addtnl_where ""

    # lets determine the content type and query from that view
    if {[info exists additional_properties]} {
        if {[db_0or1row get_1revision {} -column_array one_revision]} {
            set content_type [bcms::item::get_content_type -item_id $one_revision(item_id)]
            set table_name [bcms::get_table_name -object_type $content_type]

            set addtnl_select ", x.[join $additional_properties ", x."]"
            set addtnl_from ", ${table_name}x x"
            set addtnl_where "and r.revision_id = x.revision_id"
        } else {
            error "unable to determine content type"
        }
    }

    if {[db_0or1row get_1revision {} -column_array one_revision]} {
        return [array get one_revision]
    } else {
        return 0
    }

}

ad_proc -public bcms::revision::set_revision {
    {-revision_id:required}
    {-title}
    {-description}
    {-content}
    {-upload_file}
    {-mime_type}
} {
    Edit a revision's title, description, mime type, content and binary content

    @param revision_id   revision which to edit
    @param title         set to this title
    @param description   set to this description
    @param content       set to this content
    @param upload_file   set to this file
    @param mime_type     set to this mime_type
} {

    set update_list [list]

    if [info exists title] {
        lappend update_list "title = :title"
    }
    if [info exists description] {
        lappend update_list "description = :description"
    }
    if [info exists content] {
        lappend update_list "content = :content"
    }
    if [info exists mime_type] {
        lappend update_list "mime_type = :mime_type"
    }

    db_transaction {
        if {[info exists upload_file]} {
            # if we are uploading a file, then overwrite the existing one
            array set one_revision [bcms::revision::get_revision -revision_id $revision_id]
            set item_id $one_revision(item_id)

            # create the file in the file system and get its name and size
            set upload_tmpfile [template::util::file::get_property tmp_filename $upload_file]
            set tmp_filename [cr_create_content_file $item_id $revision_id $upload_tmpfile]
            set tmp_filesize [cr_file_size $tmp_filename]	
            if {![info exists content]} {
                # we to make sure that we do not double put the content field
                lappend update_list "content = :content"
            }
            set content $tmp_filename
            lappend update_list "content_length = :tmp_filesize"
            if {![info exists mime_type]} {
                set mime_type [template::util::file::get_property mime_type $upload_file]
                lappend update_list "mime_type = :mime_type"
            }
        }
        set update_string [join $update_list ","]
        db_dml update_revision {}
    }

}

ad_proc -public bcms::revision::delete_revision {
    {-revision_id:required}
} {
    delete a revision with revision_id
    will only delete the revision if there are
    2 or more revisions.  Will not delete if there
    is only one revision.  You can try bcms::item::delete_item
    if you want to remove an item.

    @param revision_id    id of the revision to delete

    @return 1 if the deletion is successful, 0 if not.
} {
    set revision_count [db_string get_revision_count {}]
    if {$revision_count > 1} {
        db_exec_plsql delete_revision {}
        return 1
    } else {
        return 0
    }
}

# TODO: put in the other status of expired and ready
ad_proc -public bcms::revision::set_revision_status {
    {-revision_id:required}
    {-status:required}
} {
    sets a revision to live or production, normally the query to know what to publish
    is the revision that is set to live

    @param revision_id     revision id to change the status
    @param status          the the status to this value, currently accepted are live and production
} {
    switch -exact $status {
        "live" {db_exec_plsql set_live {}}
        "production" {db_exec_plsql unset_live {}}
        default { 
            # abort the proc
            ns_log debug "bcms: revision status is unknown" 
            error "setting to a unknown status"
            return
        }
    }

}


ad_proc -public bcms::revision::copy_revision {
    {-revision_id:required}
    {-creation_user_id}
    {-creation_ip}
} {
    creates another revision based from the revision_id passed.

    @param revision_id     id of which the version will be copied from
    @param creation_user   user performing the copy
    @param creation_ip     ip address of the user performing the copy

    @return the version id of the new revision
} {

    if {![info exists creation_user_id]} { set creation_user_id [ad_conn user_id] }
    if {![info exists creation_ip]} { set creation_ip [ad_conn peeraddr] }

    return [db_exec_plsql copy_revision {}]

}
