ad_library {

	Procs for content_template, and some helper procs

}

namespace eval bcms::template {}

ad_proc -public bcms::template::create_template {
    {-template_name:required}
    {-parent_id:required}
    {-creation_user_id}
    {-creation_ip}
} {
    This proc creates a template and returns the item_id of the new template

    @param template_name           name of the template
    @param parent_id               folder id of where to store the template
    @param creation_user_id        user_id creating this item
    @param creation_user_ip        ip address which this item is created

    @return the item_id of the new template
} {
    if {![info exists creation_user_id]} { set creation_user_id [ad_conn user_id] }
    if {![info exists creation_ip]} { set creation_ip [ad_conn peeraddr] }

    set item_id [db_exec_plsql create_template {}]
    return $item_id
}

ad_proc -public bcms::template::add_template {
    {-template_id:required}
    {-title:required}
    {-description ""}
    {-content ""}
    {-creation_user_id}
    {-creation_ip}
} {
    adds a template revision

    @param template_id             item id of the template you want to add a new revision to
    @param title                   title of the template
    @param description             description of the template
    @param content                 content of the template
    @param creation_user_id        user_id creating this item
    @param creation_ip        ip address which this item is created

    @return the revision_id of the new template revision
    
} {
    if {![info exists creation_user_id]} { set creation_user_id [ad_conn user_id] }
    if {![info exists creation_ip]} { set creation_ip [ad_conn peeraddr] }

    set revision_id [db_exec_plsql add_template {}]
    return $revision_id
}

ad_proc -public bcms::template::list_templates {
    {-parent_id:required}
    {-revision latest}
    {-multirow_name bcms_list_templates}
    {-return_list:boolean}
} {
    list the template under a folder, since templates a bit different we can't use
    the general item::list_items call

    @param parent_id            id of the which contents you wish to list, normally a folder
    @param multirow_name        you can supply your own multirow name
    @param return_list          if supplied a list of ns_sets otherwise it
                                returns a multirow uses the multirow_name if given
     
    @return rows of content items below the parent_id
    with the following columns:

    template_id, name, live_revision, latest_revision, publish_status, content_type, storage_type, tree_sortkey,
    title, description

} {

    if {[string equal $revision "latest"]} { 
        set revision "i.latest_revision"
    } elseif {[string equal $revision "live"]} {
        set revision "i.live_revision"
    } else {
        error "invalid revision value, valid values are live or latest"
    }

    if {$return_list_p} {
        # return a list of lists since its not easy to pass multirow from proc to proc
        return [db_list_of_ns_sets list_templates {}]
    } else {
        # make a multirow, it is likely that this proc is not being called from another proc
        db_multirow $multirow_name list_templates {}
    }

}


ad_proc -public bcms::template::delete_template {
    {-template_id:required}
} {
    Delete a template

    @param template_id   item id of the template that will be deleted
} {
    return [db_exec_plsql delete_template {}]
}

# for possible contribution 
ad_proc -public bcms::template::get_cr_root_template_folder {} {
    gets the folder_id of the template root of cr

    @return the folder id of the root CR template
} {
    set template_root [db_exec_plsql get_template_root {}]
    return $template_root
}

ad_proc -public bcms::template::apply_template {
    {-item_id:required}
    {-template_id:required}
    {-context}
} {
    applies a template to a content item with the context given or the current
    context of BCDS

    @param item_id         item you want to apply a template
    @param template_id     the template you want to use for the item
    @param context         context of which the template is used
} {

    if ![info exists context] {
	set context [parameter::get -parameter context]
    }

    db_exec_plsql apply_template {}

}

ad_proc -public bcms::template::register_template {
	{-content_type:required}
	{-template_id:required}
	{-context} 
	{-is_default_p "f"}
} {
	Registers a template for a content_type within specified context

	@param content_type     content_type template is for
	@param tempalte_id      the template to use
	@context                context to use template
} {
	if {![exists_and_not_null context]} {
		set context [parameter::get -parameter context]
	}

	db_exec_plsql register_template {}
}
	
ad_proc -public bcms::template::unregister_template {
    {-item_id:required}
    {-template_id ""}
    {-context ""}
} {
    unregister a template for a item_id

    @param item_id         item you want to remove a template
    @param template_id     the template you want to unregister
    @param context         context of which the template
} {
    db_exec_plsql unregister_template {}
}

ad_proc -public bcms::template::get_template {
    {-item_id:required}
    {-context}
} {
    gets the template of the content item on the context specified

    @param item_id      item you want to get a template
    @param context      the context of the template

    @return an array that contains properties of the template, if no
    template is found it returns a blank array.
} {
    if ![info exists context] {
        set context [parameter::get -parameter context]
    }
    if {[db_0or1row get_template {} -column_array one_template]} {
        return [array get one_template]
    } else {
        return {}
    }

}






