ad_library {

	Provide functions for bcms common widgets
	
	@author Jun Yamog
	@creation-date 3 Mar 2002
	@cvs-id $Id: bcms-widget-procs.tcl,v 1.11 2004/02/17 14:38:25 lars Exp $
}

namespace eval bcms::widget {}

ad_proc -public bcms::widget::option_list {
    -list_of_ns_sets:required
    -value_column:required
    -display_columns:required
    {-prefix "&nbsp;"}
} {
    @depracated use patched bcms::widget::extract_values
    
    creates a list for use of ad_form's option_list.  Gets a list of ns_sets.

    -value_column is the column that will be used the html value
    -display_columns is the column or columns that will be 
    displayed in the select widget

    -prefix is appended before the display column in case the items 
    on the list_of_ns_sets have different levels.

} {

    # construct the option list
    set option_list [list]
    foreach one_row $list_of_ns_sets {
        set parent_level [ns_set get $one_row parent_level]
        set level [ns_set get $one_row level]
        set v_column [ns_set get $one_row $value_column]

        # build the display string
        set d_column ""
        foreach one_d_column $display_columns {
            append d_column "[ns_set get $one_row $one_d_column] "
        }

        #check to see if there is a heirachy... eg. level and parent_level
        if {![string equal $parent_level ""] || ![string equal $level ""]} {
            set option_list_padding [string repeat $prefix [expr ($level - $parent_level) * 4]]
        } else {
            set option_list_padding ""
        }

        # build a single select row
        lappend option_list [list "$option_list_padding $d_column" $v_column]
    }
    return $option_list

} 


ad_proc -public bcms::widget::option_tree {
    -list_of_ns_sets:required
    -value_column:required
    -display_columns:required
    {-separator " > "}
} {
    NOTE maybe candidate for removal

    makes and option list for ad_form option_list.  makes a trail on each
    select item until the -value_column changes.

    ex.  parent > item
         parent > parent > item 

} {
    # initialize the needed vars
    set option_list [list]
    set last_item ""
    set trail_string ""

    foreach one_row $list_of_ns_sets {

        set current_item [ns_set get $one_row $value_column]

        # build the display string
        set current_d_column ""
        foreach one_d_column $display_columns {
            append current_d_column "[ns_set get $one_row $one_d_column] "
        }

        if {$current_item == $last_item} {
            # the item is still the same then just build the trail for a single
            # row of select widget
            append trail_string "${separator}${current_d_column}"
        } else {
            # build a single row in the select widget and reinitialize trail_string
            if ![string equal $trail_string ""] { 
                # need to check this so the first iteration does not yeild a blank row
                lappend option_list [list $trail_string $last_item] 
            }
            set trail_string $current_d_column
        }

        # save the current item for comparison on the next iteration
        set last_item $current_item
    }

    if {[info exists current_item]} {
        # we need to append the last row
        lappend option_list [list $trail_string $current_item] 
    }

    return $option_list

}


ad_proc -public bcms::widget::extract_values {
    -list_of_ns_sets:required
    -keys_to_extract:required
} {
    Gets the values of list_of_ns_sets and creates a list of list
    {{value1 value2 ... valueX} {value1 value2 ... valueX}}

    @param -list_of_ns_sets a list of ns_sets normally created from db_list_of_ns_sets
    @param -keys_to_extract a list of keys, the value of the key present in the list
    will be returned on the results

} {

    set result_list [list]

    foreach set $list_of_ns_sets {
        set inner_list [list]
        foreach key $keys_to_extract {
            lappend inner_list [ns_set get $set $key]
        }
        lappend result_list $inner_list
    }

    return $result_list

}


ad_proc -public bcms::widget::item_context {
    {-item_id:required}
    {-root_id}
    {-root_url ""}
} {
    creates a list based the folder it resides that can be used as context or trail.

    @param item_id       id of which item to get a context
    @param root_id       start from which id
    @param root_url
    @returns a list in {{url label} {url label}} format
} {

    if {[empty_string_p $root_url]} {
	set root_url [ad_conn package_url]
    }
    if {![info exists root_id]} {
        set root_id [bcms::folder::get_bcms_root_folder]
    }

    return [bcms::widget::extract_values -list_of_ns_sets \
                [bcms::item::get_item_path -item_id $item_id -root_id $root_id -prepend_path "$root_url" -return_list] \
                -keys_to_extract {path title}]
}


ad_proc -public bcms::widget::category_context {
    {-category_id:required}
    {-root_id}
    {-root_url ""}
} {
    creates a list category listing from the root_id to the category_id
 
    @param category_id   id of which item to get a context
    @param root_id       start from which id

    @return a list in {{url label} {url label}} format
} {
    if {[empty_string_p $root_url]} {
    	set root_url [ad_conn package_url]
    }
    if {![info exists root_id]} {
         # if the root_id does not exists then start from the root
        array set root [bcms::category::get_root -category_id $category_id]
        if {[array size root] > 0} {
            set root_id $root(category_id)
        } else {
            ns_log notice "bcms: category does not exists"
        }
    }

    set root_url [ad_conn package_url]
    set root_url [string range $root_url 0 [expr [string length $root_url] - 2]]

    return [bcms::widget::extract_values -list_of_ns_sets \
                [bcms::category::get_category_path -category_id $category_id -root_id $root_id -prepend_path "$root_url" -return_list] \
                -keys_to_extract {path heading}]
}

