ad_library {

    Classified Ads Procs (Categories)

    @author Deds Castillo (deds@infiniteinfo.com)
    @creation-date 2002-10-08
    @cvs-id $Id: categories-procs.tcl,v 1.7 2004/02/26 15:03:08 jeffd Exp $

}

namespace eval classified-ads::categories {}


ad_proc -public classified-ads::categories::new {
	{-keyword_id ""}
	{-heading:required}
	{-description ""}
	{-parent_id ""}
} {

	Creates a new keyword that's used for categorization

	@param keyword_id The keyword id for this new keyword. A new id will be provided if null.
	@param heading The heading of the new keyword
	@param description The description of the new keyword
	@param parent_id The parent of this new keyword if it is a subcategory

} {
	set creation_user [ad_conn user_id]
	set creation_ip [ad_conn peeraddr]
	return [db_exec_plsql create_keyword {}]
}


ad_proc -public classified-ads::categories::edit {
	{-keyword_id:required}
	{-heading:required}
	{-description ""}
} {

	Edits properties of an existing category

	@param keyword_id The keyword id of the category to edit
	@param heading The new heading to use
	@param description The new description to use

} {
	db_transaction {
		set_heading -keyword_id $keyword_id -heading $heading
		set_description -keyword_id $keyword_id -description $description
	} on_error {
		ad_return_complaint 1 "<li>the following error occured: $errmsg"
		ad_script_abort
	}

	return $keyword_id
}


ad_proc -public classified-ads::categories::delete {
	{-keyword_id:required}
} {
	Deletes a category

	@param keyword_id The keyword id of the category to delete
} {
	return [db_exec_plsql delete_category {}]
}

ad_proc -public classified-ads::categories::get_package_keyword_id {
	{-package_id ""}
} {
	Returns the keyword_id of the associated category for this package

	@param package_id The package id to query, defaults to current package id if not specified
} {
	set main_keyword_id [classified-ads::get_main_keyword_id]

	if {!$main_keyword_id} {
		ad_return_error "Main Category Not Found" "The sytem can't find the main [classified-ads::package_key] category.  Are you sure you have restarted the server?"
	}

	if {[empty_string_p $package_id]} {
		set package_id [ad_conn package_id]
	}

	set heading "[classified-ads::package_key]-package-$package_id"
	set package_keyword_id [db_string get_id {} -default 0]

	if {!$package_keyword_id} {
		set package_keyword_id [classified-ads::categories::new -heading $heading \
			-description "Package Main Category" \
			-parent_id $main_keyword_id \
		]
	}

	return $package_keyword_id
}

ad_proc -public classified-ads::categories::get {
	{-keyword_id:required}
	{-array:required}
} {
	Returns the data for a particular category

	@param keyword_id The keyword id of the category to get the data
	@param array An array where the data will be stored
} {
	upvar $array row

	set row(heading) [get_heading -keyword_id $keyword_id]
	set row(description) [get_description -keyword_id $keyword_id]

	return 0
}

ad_proc -public classified-ads::categories::get_all_tree {
	{-keyword_id ""}
	{-include_description:boolean "0"}
	{-indent_factor 2}
	{-indent_pattern " "}
} {
	Returns a list of lists with the tree of categories for the
	passed keyword_id, indented <tt>indent_factor</tt> times by <tt>indent_pattern</tt>.

	@author Roberto Mello

	@param keyword_id The id of the keyword to get categories for.
	                  Defaults to the main keyword_id of the current package instance.

	@param include_description If especified, the category descriptions will be added to the
				   returned list.
	@param indent_factor Number of time to repeat the indentation pattern.
	@param indent_pattern A character to repeat in the indentation.

	@see classified-ads::categories::get
	@see classified-ads::categories::get_package_keyword_id
} {
	if {[empty_string_p $keyword_id]} {
		set keyword_id [classified-ads::categories::get_package_keyword_id]
	}

	set extra_fields ""
	
	if {$include_description_p} {
		set extra_fields ", subtree.description"
	}

	return [db_list_of_lists get {}]
}

ad_proc -public classified-ads::categories::get_heading {
	{-keyword_id:required}
} {
	Returns the heading for a particular category

	@param keyword_id The keyword id of the category to get the heading
} {
	return [db_exec_plsql get_heading {}]
}

ad_proc -public classified-ads::categories::set_heading {
	{-keyword_id:required}
	{-heading:required}
} {
	Sets the heading for a particular category

	@param keyword_id The keyword id of the category to set the heading
	@param heading The new heading

} {
	return [db_exec_plsql set_heading {}]
}

ad_proc -public classified-ads::categories::get_description {
	{-keyword_id:required}
} {
	Returns the description for a particular category

	@param keyword_id The keyword id of the category to get the description
} {
	return [db_exec_plsql get_description {}]
}

ad_proc -public classified-ads::categories::set_description {
	{-keyword_id:required}
	{-description ""}
} {
	Sets the description for a particular category

	@param keyword_id The keyword id of the category to set the description
	@param description The new description

} {
	return [db_exec_plsql set_description {}]
}

ad_proc -public classified-ads::categories::get_path {
	{-keyword_id:required}
} {
	Returns the path of a particular category

	@param keyword_id The keyword id of the category to get the path

} {
	return [db_exec_plsql get_path {}]
}

ad_proc -public classified-ads::categories::get_custom_fields { 
	{-keyword_id:required}
} {
	Returns a list (of lists) of column names and their pretty names of custom 
	fields available for this category. Either the field was assigned 
	directly to this category or to one of its parents.

	@author Roberto Mello
	@creation-date 2003-07-29
	@see classified-ads::ads::get_assigned_ads_by_category
} {
	return [db_list get_fields {}]
}

ad_proc -public classified-ads::categories::get_custom_fields_and_datatypes { 
	{-keyword_id:required}
} {
	Returns a list (of lists) of column names and their datatypes of custom 
	fields available for this category. Either the field was assigned 
	directly to this category or to one of its parents.

	@author Roberto Mello
	@creation-date 2003-07-29
} {
	#
	# RBM: FIXME: Hmmm, this API needs refactoring. I added this function
	#      to "make up" for another, old one.
	#

	return [db_list get_fields {}]
}

ad_proc -public classified-ads::categories::generate_trail {
	{-keyword_id:required}
	{-append_element}
} {
	Generates a cookie trail for a particular category.

	@param keyword_id The keyword id that defines the end of the trail 
			to be generated
	@param append_element The last element to put in the trail. If none, 
			then the element defined by keyword_id is used
} {
	set package_keyword_id [classified-ads::categories::get_package_keyword_id]
	set trail_list [list]

	db_foreach get_trail {} {
		if {[string match $keyword_id $package_keyword_id]} {
			set heading "Categories"
			set url "index"
		} else {
			set url "index?keyword_id=$keyword_id"
		}

		lappend trail_list [list $url $heading]
	}

	if {[exists_and_not_null append_element]} {
		lappend trail_list $append_element
	} else {
		# no extra elements were passed, so our last element should have no link
		set last_element_index [expr [llength $trail_list] - 1]
		set trail_list [lreplace $trail_list $last_element_index $last_element_index $heading]
	}
	return $trail_list
}


ad_proc -public classified-ads::categories::item_assign {
	{-keyword_id:required}
	{-item_id:required}
} {

	Maps a content item to a keyword

	@param keyword_id The keyword id of the category we map to
	@param item_id The item id of the content item we map

} {
	return [db_exec_plsql add_map {}]
}


ad_proc -public classified-ads::categories::item_unassign {
	{-keyword_id:required}
	{-item_id:required}
} {

	Unmaps a content item to a keyword

	@param keyword_id The keyword id of the category we unmap to
	@param item_id The item id of the content item we unmap

} {
	return [db_exec_plsql delete_map {}]
}
