ad_library {

    Classified Ads Procs (Widgets)

    @author Deds Castillo (deds@infiniteinfo.com)
    @creation-date 2002-10-08
    @cvs-id $Id: widget-procs.tcl,v 1.8 2003/07/31 09:31:00 rmello Exp $

}

namespace eval classified-ads::widgets {
    #
    # DEDS: FIXME - check also the datatype in acs_attributes, 
    #               we may be doing it wrong.

    ad_proc -public get_widget_params {
        {-content_type:required}
        {-form_id:required}
	{-keyword_id ""}
    } {
        
        Returns the widgets for a particular content_type.
	If the keyword_id is passed, only widgets belonging to that category
	will be returned, otherwise widgets available to all categories are
	returned.
        
        @param content_type The content type whose widgets we are interested in
        @param form_id The id of the form where we create the widgets
        @param keyword_id The keyword_id of the category for which we want widgets.
    } {
        
        set package_id [ad_conn package_id]

	if { [empty_string_p $keyword_id] } {
		#
		# keyword_id was not passed, so we get the keyword_id for this
		# package instance.
		#

		set keyword_id [classified-ads::categories::get_package_keyword_id \
					-package_id $package_id]
	}

        set widget_list [classified-ads::get_attributes -keyword_id $keyword_id]

        foreach one_widget $widget_list {
            set extra_ops ""
            set widget_command "template::element::create $form_id [ns_set get $one_widget attribute_name] \
                -label \"[ns_set get $one_widget label]\" \
                -datatype \"[ns_set get $one_widget datatype]\" \
                -widget \"[ns_set get $one_widget widget]\""

            # Were html options passed?

            set html_options [ns_set get $one_widget html_options]

            if {![empty_string_p $html_options]} {
                append extra_ops " -html $html_options"
            }

            # Is there a default value?

            #set default_value [ns_set get $one_widget default_value]
	    
            #if {![empty_string_p $default_value]} {
            #    append extra_ops " -value $default_value"
            #}

            # Is there a validate clause?

            set validate_clause [ns_set get $one_widget validate]

            if {![empty_string_p $validate_clause]} {
                append extra_ops " -validate $validate_clause"
            }

            # Is it optional?

            if {[string equal [ns_set get $one_widget optional_p] "t"]} {
                append extra_ops " -optional"
            }

            # If it has select options

	    set options_clause [ns_set get $one_widget select_options]

	    if {![empty_string_p $options_clause]} {
                append extra_ops " -options " 
	        append extra_ops $options_clause
	    }
            #
	    # RBM: Did the user pass extra form builder options?
	    #
	    # This is somewhat dangerous, but the admin must know what
	    # s/he is doing when entering extra options. It was the only 
	    # way I could think of to allow for different options like
	    # -format, -help_text, -help, etc. to be added on a
	    # per-field basis.
	    #

	    set extra_clause [ns_set get $one_widget extra_options]
	    
	    if { ![empty_string_p $extra_clause] } {
	        append extra_ops " $extra_clause"
            }

            append widget_command $extra_ops
	    ns_log notice $widget_command

            eval $widget_command
        }

        return 0
    }


   ad_proc -public set_widget_values {
       {-form_id:required}
       {-form_values:required}
    } {
        Assigns values to the elements of a form

        @param form_id The id of the form that we assign values to
        @param form_values A list of lists containing the values to assign to.
                           Each sublist must have two elements with the first being the
                           element name and the second the element value.
    } {
        foreach one_pair $form_values {
            template::element::set_properties $form_id [lindex $one_pair 0] -value [lindex $one_pair 1]
        }

        return 0
    }


   ad_proc -public date_widget_to_sql {
       {-date:required}
    } {
        Converts the contents of a date widget into a format understood by sql

        @param date the date value as passed by the widget
    } {
        set year [template::util::date::get_property year $date]
        set month [template::util::date::get_property month $date]
        set day [template::util::date::get_property day $date]

        # put together the sql date
        set sql_date_string "$year-$month-$day"
        if {[string match $sql_date_string "--"]} {
            set sql_date_string ""
        }
        return $sql_date_string

    }


    ad_proc -public sql_to_date_widget {
        {-sql_date:required}
        {-format:required}
    } {
        converts an sql date to those accepted by our widget

        @param sql_date the sql date
        @param format the format to convert to.  can be any of YYYY-MM-DD,
                      HH12:MIam, or HH24:MI
    } {
        # stolen from calendar procs

        set date [template::util::date::create]

        switch -exact -- $format {
            {YYYY-MM-DD} {
                regexp {([0-9]*)-([0-9]*)-([0-9]*)} $sql_date all year month day

                set date [template::util::date::set_property format $date {DD MONTH YYYY}]
                set date [template::util::date::set_property year $date $year]
                set date [template::util::date::set_property month $date $month]
                set date [template::util::date::set_property day $date $day]
            }

            {HH12:MIam} {
                regexp {([0-9]*):([0-9]*) *([aApP][mM])} $sql_date all hours minutes ampm
                
                set date [template::util::date::set_property format $date {HH12:MI am}]
                set date [template::util::date::set_property hours $date $hours]
                set date [template::util::date::set_property minutes $date $minutes]                
                set date [template::util::date::set_property ampm $date [string tolower $ampm]]
            }

            {HH24:MI} {
                regexp {([0-9]*):([0-9]*)} $sql_date all hours minutes

                set date [template::util::date::set_property format $date {HH24:MI}]
                set date [template::util::date::set_property hours $date $hours]
                set date [template::util::date::set_property minutes $date $minutes]
            }

            {HH24} {
                set date [template::util::date::set_property format $date {HH24:MI}]
                set date [template::util::date::set_property hours $date $sql_date]
                set date [template::util::date::set_property minutes $date 0]
            }
        }

        return $date
    }

}
