set pretty_name [_ acs-subsite.subsite]
set pretty_plural [_ acs-subsite.subsites]

set admin_p [permission::permission_p -object_id [ad_conn package_id] -privilege admin -party_id [ad_conn untrusted_user_id]]

template::list::create \
    -name subsites \
    -multirow subsites \
    -no_data {<p>No communities exist yet.  Request one!</p>} \
    -main_class communities \
    -selected_format list \
    -formats {
        table { 
            label Table
            layout table
        }
        list { 
            label List
            layout list
            template {
                <h2><listelement name="instance_name"></h2>
                <p class="blurb"><listelement name="blurb"></p>
                <p class="activity">34 items this week, last update 20/4/2004</p>
                <p class="member-state"><listelement name="member_state"></p>
            }
        }
    } -elements {
        instance_name {
            label "[_ acs-subsite.Name]"
            link_url_eval "$url/"
        }
        num_members {
            label "[_ acs-subsite.Members]"
            html { align right }
        }
        member_state {
            label "Member state"
            display_template {
                <switch @subsites.member_state@>
                  <case value="approved">Member</case>
                  <case value="needs approval">Membership requested</case>
                  <case value="rejected">Membership denied</case>
                  <default>
                    @subsites.member_state@
                    <if @subsites.join_policy@ eq "open"><a href="@subsites.join_url@" class="button">Join</a></if>
                    <else><a href="@subsites.join_url@" class="button">Request membership</a></else>
                  </default>
                </switch>
            }
        }
        blurb { 
            hide_p 1
            display_template {@subsites.blurb;noquote@}
        }
    }


set subsite_node_id [subsite::get_element -element node_id]
set subsite_url [subsite::get_element -element url]

set untrusted_user_id [ad_conn untrusted_user_id]

db_multirow -extend { join_url blurb } subsites select_subsites {
    select p.package_id,
           p.instance_name,
           n.node_id, 
           n.name,
           :subsite_url || n.name as url,
           (select count(*)
            from   group_approved_member_map m
            where  m.rel_type = 'membership_rel'
            and    m.group_id = ag.group_id) as num_members,
           (select min(r2.member_state)
            from   group_member_map m2,
                   membership_rels r2
            where  m2.group_id = ag.group_id
            and    m2.member_id = :untrusted_user_id
            and    r2.rel_id = m2.rel_id) as member_state,
           g.group_id,
           g.join_policy
    from   site_nodes n,
           apm_packages p,
           application_groups ag,
           groups g
    where  n.parent_id = :subsite_node_id
    and    p.package_id = n.object_id
    and    p.package_key = 'acs-subsite'
    and    ag.package_id = p.package_id
    and    g.group_id = ag.group_id
    and    (exists (select 1 
                   from   all_object_party_privilege_map perm 
                   where  perm.object_id = p.package_id
                   and    perm.privilege = 'read'
                   and    perm.party_id = :untrusted_user_id) or g.join_policy != 'closed')
    order  by lower(instance_name)
} {
    set join_url [export_vars -base "${subsite_url}register/user-join" { group_id { return_url [ad_return_url] } }]
    # need to get the blurb text for each community
    set cop_id [site_node::get_children -node_id $node_id -package_key cop-ui -element package_id]
    set blurb [parameter::get -parameter Description -package_id $cop_id]
    set blurb_format [parameter::get -parameter DescriptionFormat -package_id $cop_id]
}
