ad_library {
    Procs for the mailing list manager package.

    @author Timo Hentschel (thentschel@sussdorff-roy.com)

    @creation-date 16 April 2003
    @cvs-id $Id:
}

namespace eval mailing_list {

    ad_proc -public add {
	{-list_id ""}
	{-package_id ""}
	{-name ""}
	{-locale ""}
	{-teaser ""}
	{-sender_email ""}
	{-confirm_subject ""}
	{-confirm_body ""}
	{-confirm_mime_type "text/plain"}
	{-welcome_subject ""}
	{-welcome_body ""}
	{-welcome_mime_type "text/plain"}
	{-remind_subject ""}
	{-remind_body ""}
	{-remind_mime_type "text/plain"}
	{-expiration_date [db_null]}
	{-first_reminder 7}
	{-second_reminder 0}
	{-comments ""}
    } {
	Creates a new mailing-list
    } {
	set creation_ip [ad_conn peeraddr]
	set user_id [ad_conn user_id]
	if {[empty_string_p $package_id]} {
	    set package_id [ad_conn package_id]
	}

	db_transaction {
	    set list_id [db_exec_plsql insert_mailing_list {}]

	    db_dml set_mailing_list_clobs {}
	}

	return $list_id
    }

    ad_proc -public edit {
	{-list_id:required}
	{-name ""}
	{-locale ""}
	{-teaser ""}
	{-sender_email ""}
	{-confirm_subject ""}
	{-confirm_body ""}
	{-confirm_mime_type "text/plain"}
	{-welcome_subject ""}
	{-welcome_body ""}
	{-welcome_mime_type "text/plain"}
	{-remind_subject ""}
	{-remind_body ""}
	{-remind_mime_type "text/plain"}
	{-expiration_date [db_null]}
	{-first_reminder 7}
	{-second_reminder 0}
	{-comments ""}
    } {
	Edits mailing-list
    } {
	set modifying_ip [ad_conn peeraddr]
	set user_id [ad_conn user_id]

	db_transaction {
	    db_exec_plsql update_mailing_list {}

	    db_dml set_mailing_list_clobs {}
	}
    }

    ad_proc -public user_subscribed_p {
	-list_id:required
	-user_id:required
    } {
	Checks if a user is already subscribed to a list
    } {
	return [db_0or1row check_user_entry_exists {}]
    }

    ad_proc -public add_user {
	-list_id:required
	-user_id:required
	{-confirmed_p t}
    } {
	Add a user to a mailing list
    } {
	# try to add new user map entry
	# if it fails, the user is already subscribed or unsubscribed by his own choice
	if {![user_subscribed_p -list_id $list_id -user_id $user_id]} {
	    db_dml add_user_entry {}
	    catch {
		db_dml add_user_log_entry {}
	    }
	    return 1
	} else {
	    return 0
	}
    }

    ad_proc -private send_reminder {} {
	Sends a reminder email to users not having confirmed the
	mass subscribtion
    } {
	set reminders [db_list_of_lists get_unconfirmed_subscriptions {}]

	# send reminder
	ad_conn -set package_key mailing-lists
	foreach reminder $reminders {
	    util_unlist $reminder user_id sender_email remind_subject remind_body remind_mime_type mime_charset list_id node_id package_id

	    ad_conn -set package_id $package_id
	    set confirmation_link "[ad_url][site_node::get_url -node_id $node_id]confirm?[export_url_vars list_id user_id]"

	    mailing_list::util::send_mail -user_id $user_id -from_email $sender_email -subject $remind_subject -body $remind_body -mime_type $remind_mime_type -charset $mime_charset -link $confirmation_link

	    ns_log Notice "List subscription reminder sent to user $user_id"

	    # log sending the reminder mail
	    db_dml log_reminder_sending {}
	}
    }
}

namespace eval mail_class {

    ad_proc -public add {
	{-mail_class_id ""}
	{-package_id ""}
	{-name ""}
	{-locale ""}
	{-sender_email ""}
	{-subject ""}
	{-subject_change_p t}
	{-text_header ""}
	{-text_header_change_p t}
	{-text_body ""}
	{-text_body_change_p t}
	{-text_footer ""}
	{-text_footer_change_p t}
	{-html_header ""}
	{-html_header_change_p t}
	{-html_body ""}
	{-html_body_change_p t}
	{-html_footer ""}
	{-html_footer_change_p t}
	{-mime_type "text/plain"}
	{-comments ""}
    } {
	Creates a new mail-class
    } {
	set creation_ip [ad_conn peeraddr]
	set user_id [ad_conn user_id]
	if {[empty_string_p $package_id]} {
	    set package_id [ad_conn package_id]
	}

	db_transaction {
	    set mail_class_id [db_exec_plsql insert_mail_class {}]

	    db_dml set_mail_class_clobs {}
	}

	return $mail_class_id
    }

    ad_proc -public edit {
	{-mail_class_id:required}
	{-name ""}
	{-locale ""}
	{-sender_email ""}
	{-subject ""}
	{-subject_change_p t}
	{-text_header ""}
	{-text_header_change_p t}
	{-text_body ""}
	{-text_body_change_p t}
	{-text_footer ""}
	{-text_footer_change_p t}
	{-html_header ""}
	{-html_header_change_p t}
	{-html_body ""}
	{-html_body_change_p t}
	{-html_footer ""}
	{-html_footer_change_p t}
	{-mime_type "text/plain"}
	{-comments ""}
    } {
	Edits mail-class
    } {
	set modifying_ip [ad_conn peeraddr]
	set user_id [ad_conn user_id]

	db_transaction {
	    db_exec_plsql update_mail_class {}

	    db_dml set_mail_class_clobs {}
	}
    }
}

namespace eval mail_job {

    ad_proc -public add {
	{-class_id:required}
	{-list_id:required}
	{-selection_id:required}
	{-locale ""}
	{-sender_email ""}
	{-track_links_p f}
	{-subject ""}
	{-text_header ""}
	{-text_body ""}
	{-text_footer ""}
	{-html_header ""}
	{-html_body ""}
	{-html_footer ""}
	{-template_p f}
	{-mime_type "text/plain"}
	{-state active}
	{-scheduled_date [db_null]}
	{-bind_vars ""}
    } {
	Creates a new mail-job
    } {
	set creation_ip [ad_conn peeraddr]
	set user_id [ad_conn user_id]
	set package_id [ad_conn package_id]

	db_transaction {
	    set mail_job_id [db_exec_plsql insert_mail_job {}]

	    db_dml set_email_clobs {}

	    if {![empty_string_p $selection_id]} {
		db_1row get_users_selection_query {}

		db_dml set_selection_query {}
	    }
	}

	return $mail_job_id
    }

    ad_proc -public edit {
	{-mail_job_id:required}
	{-locale ""}
	{-sender_email ""}
	{-track_links_p f}
	{-subject ""}
	{-text_header ""}
	{-text_body ""}
	{-text_footer ""}
	{-html_header ""}
	{-html_body ""}
	{-html_footer ""}
	{-template_p f}
	{-mime_type "text/plain"}
	{-state active}
	{-scheduled_date [db_null]}
	{-bind_vars ""}
    } {
	Edits a mail-job
    } {
	set modifying_ip [ad_conn peeraddr]
	set user_id [ad_conn user_id]

	db_transaction {
	    db_exec_plsql update_mail_job {}

	    db_dml set_email_clobs {}
	}
    }

    ad_proc -private after_install {} {
	Callback to be called after package installation.
	Adds the service contract implementations for bounce-management.

	@author Timo Hentschel (thentschel@sussdorff-roy.com)
    } {
	acs_sc::impl::new -contract_name AcsMailLite -name mailing_lists -owner mailing_lists
	acs_sc::impl::alias::new -contract_name AcsMailLite -impl_name mailing_lists -operation MailBounce -alias mail_job::mail_bounced
	acs_sc::impl::binding::new -contract_name AcsMailLite -impl_name mailing_lists
    }

    ad_proc -private before_uninstall {} {
	Callback to be called before package uninstallation.
	Removes the service contract implementations for bounce-management.

	@author Timo Hentschel (thentschel@sussdorff-roy.com)
    } {
	# shouldn't we first delete the bindings?
	acs_sc::impl::delete -contract_name AcsMailLite -impl_name mailing_lists
    }
}


# procs for acsobject service contract
ad_proc -public ml_get_mailing_list_pageurl { object_id } {
    Returns the page that displays a mailing-list
} {
    return "madmin/lists?list_id=$object_id"
}

ad_proc -public ml_get_mail_job_pageurl { object_id } {
    Returns the page that displays a mail-job
} {
    return "madmin/jobs?mail_job_id=$object_id"
}

ad_proc -public ml_get_mail_class_pageurl { object_id } {
    Returns the page that displays a mail-class
} {
    return "madmin/classes?class_id=$object_id"
}
