ad_page_contract {
} {
    list_id:integer,optional
}

set user_id [auth::require_login]
set package_id [ad_conn package_id]
permission::require_permission -object_id $package_id -privilege mailing_list_admin

if {[info exists list_id]} {
    set _list_id $list_id
    set action Edit
} else {
    set _list_id 0
    set action Add
}

set context_bar [list [list "." Administration] "$action Mailing List"]

set languages [db_list_of_lists get_ad_locales {}]

ad_form -name mailing_list_form -action lists -form {
    {list_id:key}
    {name:text {label "Name"} {html {size 50 maxlength 1000}}}
    {locale:text(select) {label "Language"} {options $languages}}
    {teaser:text(textarea) {label "Teaser"} {html {rows 5 cols 80}}}
}

if {![empty_string_p [category_tree::get_mapped_trees $package_id]]} {
    category::ad_form::add_widgets -container_object_id $package_id -categorized_object_id $_list_id -form_name mailing_list_form
}

ad_form -extend -name mailing_list_form -form {
    {sender_email:text {label "Mail Sender"} {html {size 50 maxlength 1000}}}
    {confirm_subject:text {label "Confirmation Mail Subject"} {html {size 50 maxlength 1000}}}
    {confirm_body:text(textarea) {label "Confirmation Mail Body"} {html {rows 5 cols 80}}}
    {confirm_mime_type:text(select) {label "Confirmation Mail Type"} {options {{"Plain Text" text/plain} {"HTML" text/html}}}}
    {welcome_subject:text {label "Welcome Mail Subject"} {html {size 50 maxlength 1000}}}
    {welcome_body:text(textarea) {label "Welcome Mail Body"} {html {rows 5 cols 80}}}
    {welcome_mime_type:text(select) {label "Welcome Mail Type"} {options {{"Plain Text" text/plain} {"HTML" text/html}}}}
    {remind_subject:text {label "Reminder Mail Subject"} {html {size 50 maxlength 1000}}}
    {remind_body:text(textarea) {label "Reminder Mail Body"} {html {rows 5 cols 80}}}
    {remind_mime_type:text(select) {label "Reminder Mail Type"} {options {{"Plain Text" text/plain} {"HTML" text/html}}}}
    {expiration_date:date,to_sql(sql_date),to_html(sql_date) {label "Expiration Date"} {format "DD MONTH YYYY"}}
    {first_reminder:integer {label "1st Confirmation Reminder"} {html {size 5 maxlength 5}}}
    {second_reminder:integer {label "2nd Confirmation Reminder"} {html {size 5 maxlangth 5}}}
    {comments:text(textarea),optional {label "Comments"} {html {rows 5 cols 80}}}
} -new_request {
    set name ""
    set locale [ad_conn locale]
    set teaser ""
    set create_class_p t
    set sender_email "lists@greenpeace.org"
    set confirm_subject ""
    set confirm_body ""
    set confirm_mime_type "text/plain"
    set welcome_subject ""
    set welcome_body ""
    set welcome_mime_type "text/plain"
    set remind_subject ""
    set remind_body ""
    set remind_mime_type "text/plain"
    set first_reminder 7
    set second_reminder 0
    set comments ""
    db_1row get_current_date {}
} -edit_request {
    permission::require_permission -object_id $list_id -privilege admin

    db_1row get_list_data {}
} -validate {
    {confirm_body {
	[empty_string_p [mailing_list::util::check_valid_vars -text $confirm_body -valid_vars [list user_id user_password x link]]]}
	"In the confirmation email you can only use the template variables @user_id@, @user_password@, @link@ and @x@."}
    {confirm_body {
	[regexp {@link@} $confirm_body match] || [regexp {@x@} $confirm_body match]}
	"You have to use either @x@ or @link@ to provide the user a confirmation link."}
    {welcome_body {
	[empty_string_p [mailing_list::util::check_valid_vars -text $welcome_body -valid_vars [list user_id user_password link]]]}
	"In the welcome email you can only use the template variables @user_id@, @user_password@ and @link@."}
    {remind_body {
	[empty_string_p [mailing_list::util::check_valid_vars -text $remind_body -valid_vars [list user_id x link]]]}
	"In the reminder email you can only use the template variables @user_id@, @x@ and @link@."}
} -on_submit {
    set category_ids [category::ad_form::get_categories -container_object_id $package_id]
} -new_data {
    set list_id [mailing_list::add -name $name -locale $locale -teaser $teaser \
		     -sender_email $sender_email -welcome_subject $welcome_subject \
		     -welcome_body $welcome_body -welcome_mime_type $welcome_mime_type \
		     -confirm_subject $confirm_subject -confirm_body $confirm_body \
		     -confirm_mime_type $confirm_mime_type \
		     -remind_subject $remind_subject -remind_body $remind_body \
		     -remind_mime_type $remind_mime_type -expiration_date $expiration_date \
		     -first_reminder $first_reminder -second_reminder $second_reminder \
		     -comments $comments]

    if {[exists_and_not_null category_ids]} {
	category::map_object -object_id $list_id $category_ids
    }

    permission::grant -object_id $list_id -privilege admin -party_id $user_id

} -edit_data {
    permission::require_permission -object_id $list_id -privilege admin

    mailing_list::edit -list_id $list_id -name $name -locale $locale -teaser $teaser \
	-sender_email $sender_email -welcome_subject $welcome_subject \
	-welcome_body $welcome_body -welcome_mime_type $welcome_mime_type \
	-confirm_subject $confirm_subject -confirm_body $confirm_body \
	-confirm_mime_type $confirm_mime_type -remind_subject $remind_subject \
	-remind_body $remind_body -remind_mime_type $remind_mime_type \
	-expiration_date $expiration_date -first_reminder $first_reminder \
	-second_reminder $second_reminder -comments $comments

    if {[exists_and_not_null category_ids]} {
	category::map_object -remove_old -object_id $list_id $category_ids
    }
} -after_submit {
    ad_returnredirect "."
    ad_script_abort
}

ad_return_template
return
