ad_library {

    procs to manage dotlrn personal communities

    @author yon@openforce.net
    @creation-date 2002-08-23
    @version $Id: personal-community-procs.tcl,v 1.2 2005/02/26 17:52:19 jeffd Exp $

}

namespace eval dotlrn_pers_community {

    ad_proc -public community_type {} {
        return the community type
    } {
        return dotlrn_pers_community
    }

    ad_proc -public get_url_part {} {
        return the url part for this community type
    } {
        return "personal-community"
    }

    ad_proc -public get_url {} {
        return the url part for this community type
    } {
        return "/[get_url_part]"
    }

    ad_proc -public is_initialized {} {
        is dotlrn_class initialized with the right community_type?
    } {
        dotlrn_community::is_initialized -community_type [community_type]
    }

    ad_proc -public init {} {
        create base community_type for dotlrn_pers_communities
    } {
        db_transaction {
            new_type_portal -pretty_name "Personal Communities"
            
            set package_id [dotlrn::mount_package \
                -package_key [get_url_part] \
                -url [get_url_part] \
                -pretty_name "Personal Communities" \
                -directory_p t \
            ]
        
            dotlrn_community::set_type_package_id \
                -community_type [community_type] \
                -package_id $package_id
        }
    }

    ad_proc -public new_type_portal {
        {-pretty_name:required}
    } {
        (copied from dotlrn::new_type_portal)
    } {

        set csv_list {Home,Simple 2-Column;My Files,Simple 1-Column}
        set user_id -1        
        
        set portal_id [portal::create \
            -name "$pretty_name Portal" \
            -csv_list $csv_list \
            $user_id
        ]
        
        # Associate this type with portal_id, must be before applet
        # callbacks, since they use this info
        dotlrn::set_type_portal_id \
            -type [community_type] \
            -portal_id $portal_id

        # Add the default applets 
        set default_applets_list {dotlrn_dotlrn dotlrn_forums dotlrn_research}
        
        foreach applet_key $default_applets_list {
            if {[dotlrn_applet::applet_exists_p -applet_key $applet_key]} {
                dotlrn_community::applet_call $applet_key AddPortlet [list $portal_id]
            }
        }

        set args [ns_set create]
        ns_set put $args package_id 0
        ns_set put $args folder_id 0
        ns_set put $args param_action overwrite
        ns_set put $args page_name {My Files}

        dotlrn_fs::add_portlet_helper $portal_id $args

    }

    ad_proc -public new {
        {-user_id:required}
        {-pretty_name:required}
        {-description ""}
        {-join_policy closed}
        {-parent_community_id ""}
    } {
        creates a new personal community and returns the community key
        (mostly copied from dotlrn_community::new)
    } {
        set community_key [dotlrn::generate_key -name $pretty_name]

        dotlrn_community::check_community_key_valid_p \
            -complain_if_invalid \
            -community_key $community_key \
            -parent_community_id $parent_community_id

        set package_id [dotlrn::get_package_id]

        set extra_vars [ns_set create]
        ns_set put $extra_vars parent_community_id $parent_community_id
        ns_set put $extra_vars community_type [community_type]
        ns_set put $extra_vars community_key $community_key
        ns_set put $extra_vars pretty_name $pretty_name
        ns_set put $extra_vars pretty_plural $pretty_name
        ns_set put $extra_vars description $description
        ns_set put $extra_vars context_id $package_id
        ns_set put $extra_vars join_policy $join_policy
        ns_set put $extra_vars user_id $user_id

        db_transaction {

            set community_id [package_instantiate_object -extra_vars $extra_vars [community_type]]

            # YON MAJOR HACK
            # acs_object.new() initializes the acs_attributes for us if the
            # object_type of this community matches the object_type of the
            # acs_attributes. this fucks us because we use dotlrn_community
            # as the object_type for subgroups which means that their
            # attributes will be defaulted to empty strings but we will think
            # that they are set. we must delete them.
            db_dml delete_acs_attribute_values {
                delete
                from acs_attribute_values
                where object_id = :community_id
            }

            set template_id [dotlrn::get_portal_id_from_type -type [community_type]]

            # Create comm's portal page
            set portal_id [portal::create \
                -template_id $template_id \
                -name "$pretty_name Portal" \
                -context_id $community_id \
                $user_id \
            ]

            # Create the comm's non-member page
            set non_member_portal_id [portal::create \
                -name "$pretty_name Non-Member Portal" \
                -default_page_name [dotlrn::parameter -name non_member_page_name] \
                -context_id $community_id \
                $user_id \
            ]

            # Create the comm's admin page
            set admin_portal_id [portal::create \
                -name "$pretty_name Administration Portal" \
                -default_page_name [dotlrn::parameter -name admin_page_name] \
                -context_id $community_id \
                $user_id \
            ]

            # Set up the rel segments
            dotlrn_community::create_rel_segments -community_id $community_id

            # Set up the node
            if {[empty_string_p $parent_community_id]} {
                set parent_node_id [dotlrn_community::get_type_node_id [community_type]]
            } else {
                set parent_node_id [dotlrn_community::get_community_node_id $parent_community_id]
            }

            set package_id [site_node::instantiate_and_mount \
                -node_name $community_key \
                -parent_node_id $parent_node_id \
                -package_key [dotlrn_community::one_community_package_key] \
                -package_name $pretty_name \
                -context_id $community_id \
            ]

            # Set the right parameters
            ad_parameter -package_id $package_id -set 0 dotlrn_level_p
            ad_parameter -package_id $package_id -set 0 community_type_level_p
            ad_parameter -package_id $package_id -set 1 community_level_p

            # Set up the node
            dotlrn_community::set_package_id $community_id $package_id

            # update the portal_id and non_member_portal_id
            db_dml dbqd.dotlrn.tcl.community-procs.dotlrn_community::new.update_portal_ids {}

            # add the default applets based on the community type
            # 2. the the list of default applets for this type
            set default_applets_list {dotlrn_dotlrn dotlrn_forums dotlrn_research}

            foreach applet_key $default_applets_list {
                if {[dotlrn_applet::applet_exists_p -applet_key $applet_key]} {
                    dotlrn_community::add_applet_to_community $community_id $applet_key
                }
            }

            set args [ns_set create]
            ns_set put $args package_id 0
            ns_set put $args folder_id [dotlrn_fs::get_user_root_folder -user_id $user_id]
            ns_set put $args param_action overwrite
            ns_set put $args page_name {My Files}

            dotlrn_fs::add_portlet_helper $portal_id $args

            add_user -rel_type dotlrn_admin_rel -community_id $community_id -user_id $user_id

        }

        # This new community should _not_ inherit it's permissions
        # from the root dotlrn instance. Why? All dotlrn users
        # can read the root dotlrn instance, but only members of
        # this community should be able to read this instance (and
        # it's children)
        permission::set_not_inherit -object_id $community_id

        return $community_id

    }

    ad_proc -public add_user {
        {-rel_type ""}
        {-community_id:required}
        {-user_id:required}
        {-member_state approved}
    } {
        assigns a user to a particular role in the personal's community.
    } {
        if [empty_string_p $rel_type] {
            set rel_type dotlrn_member_rel
        }

        dotlrn_community::add_user_to_community \
            -rel_type $rel_type \
            -community_id $community_id \
            -user_id $user_id \
            -member_state $member_state
    }

}
