
ad_proc -private fill_subsite_tree {
    -parent_id:required
    -context_id:required
    -level:required
    -max_level:required
    -width:required
} {
    Recursively fill a subsite tree

    @param parent_id We'll root our new tree at this site node
    @param context_id The object_id of the package corresponding to parent_id
    @param level The current level we're filling
    @param max_level How many levels to fill
    @param width How many acs-subsite instances to mount at each level

} {
    for { set i 1 } { $i <= $width } { incr i } {

         # Build this node

         db_transaction {
             set package_id [db_nextval acs_object_id_seq]
             set node_id [site_node::new -name subsite_$package_id -parent_id $parent_id]
             apm_package_instance_new \
                 -package_id $package_id \
                 -instance_name subsite_$package_id \
                 -context_id $context_id \
                 -package_key acs-subsite
             site_node::mount -node_id $node_id -object_id $package_id
             site_node::update_cache -node_id $node_id
             apm_invoke_callback_proc -package_key acs-subsite -type "after-instantiate" -arg_list [list package_id $package_id]
             # apm_package_call_post_instantiation_proc $package_id acs-subsite
         }

         # Fill in the subtree

         if { $level < $max_level } {
             fill_subsite_tree -parent_id $node_id -context_id $package_id -level [expr { $level + 1 }] -max_level $max_level -width $width
         }
    }
}

ad_proc -private fill_folder_tree {
    -folder_id:required
    -level:required
    -max_level:required
    -width:required
    -files:required
    -file_name:required
    -upload_file:required
    -indb_p:required
    -user_perms:required
    -reg_user_perms:required
} {
    Recursively fill a subsite tree

    @param folder_id We'll add the new folders and files to this folder
    @param level The current level we're filling
    @param max_level How many levels to fill
    @param width How many folders to create at each level
    @param files How many filest to create in each folder

} {
    set upload_filename [template::util::file::get_property filename $upload_file]
    set tmp_filename [template::util::file::get_property tmp_filename $upload_file]
    set mime_type [cr_filename_to_mime_type -create $upload_file]

    for { set i 1 } { $i <= $width } { incr i } {

         # Build this folder

         # The new_folder proc doesn't allow us to pass in a an object_id, but we
         # need unique ids over multiple calls to the add-file script and the easiest
         # way to do this is to use an object_id.  I'll cheat and leave holes in the
         # object sequence because I don't want to touch the file-storage package (I
         # want this script to work with the recently released OpenACS 4.6 tarball)

         set folder_name folder_[db_nextval acs_object_id_seq]

         db_transaction {

             set new_folder_id [fs::new_folder -name $folder_name -pretty_name $folder_name -parent_id $folder_id]

             for { set j 1 } { $j <= $files } { incr j } {

                  set new_file_name ${folder_name}_$j
                  set item_id [db_exec_plsql new_item {}]
                  set revision_id [db_exec_plsql new_version {}]
                  db_dml set_lob_content "" -blob_files [list $file_name]
                  db_dml set_lob_size ""
                  db_exec_plsql set_live {}

                  foreach user_perm $user_perms {
                      permission::grant -party_id [ad_conn user_id] -object_id $item_id -privilege $user_perm
                  }

                  foreach reg_user_perm $reg_user_perms {
                      permission::grant -party_id -2 -object_id $item_id -privilege $reg_user_perm
                  }
             }
         } 

         # Fill in the subtree

         if { $level < $max_level } {
             fill_folder_tree -folder_id $new_folder_id -level [expr { $level + 1 }]  -max_level $max_level \
                 -width $width -files $files -file_name $file_name -upload_file $upload_file -indb_p $indb_p \
                 -user_perms $user_perms -reg_user_perms $reg_user_perms
         }
    }
}

