ad_page_contract {

    Add an arbitrary number of bugs and comments to the system

    @author Don Baccus (dhogaza@pacifier.com)
    @creation-date 2003-01-07

}

set package_id [ad_conn package_id]
set user_id [ad_conn user_id]
permission::require_permission -party_id $user_id -object_id $package_id -privilege admin

ad_form -name add_bugs -html { enctype multipart/form-data } -form {
    {new_bug_base_name:text            {label "Base name of bug?"}}
    {new_bugs_count:integer            {label "Number of new bugs?"}
                                         {html { size 5 }}}
    {new_comments_count:integer           {label "Number of comments per bug?"}
                                         {html { size 5 }}}
} -validate {
    {new_bugs_count
        { $new_bugs_count > 0 }
        "Number of new bugs must be greater than zero"
    }
    {new_comments_count
        { $new_comments_count > 0 }
        "Number of comments per bug must be greater than zero"
    }
} -on_submit {

    array set site_node [site_node::get -url /bug-tracker]

    # Don't believe the bug-tracker Tcl API that misleads you into
    # thinking that you can explicitly pass package_id as a parameter to
    # various procs.  The vile bug_tracker::conn proc guarantees this
    # does not work.

    set old_package_id [ad_conn package_id]
    ad_conn -set package_id $site_node(package_id)
    set package_id [ad_conn package_id]
    set user_id [ad_conn user_id]

    # Get a component_id, any component_id
    db_1row get_component_id {}

    for {} { $new_bugs_count > 0 } { incr new_bugs_count -1 } {
        db_1row new_bug_id {}
        bug_tracker::bug::new \
            -bug_id $bug_id \
            -package_id $package_id \
	    -component_id $component_id \
	    -found_in_version [bug_tracker::conn user_version_id] \
	    -summary ${new_bug_base_name}_summary_$new_bugs_count \
	    -description ${new_bug_base_name}_description_$new_bugs_count \
	    -desc_format text/html \
            -keyword_ids {}

    }

    ad_conn -set package_id $old_package_id
    ad_returnredirect "index"
    ad_script_abort

}
set context {"Add bugs"}

ad_return_template
