ad_page_contract {

    Add an arbitrary number of files to the system

    @author Don Baccus (dhogaza@pacifier.com)
    @creation-date 2003-01-07

}

set package_id [ad_conn package_id]
set user_id [ad_conn user_id]
permission::require_permission -party_id $user_id -object_id $package_id -privilege admin

ad_form -name add_files -html { enctype multipart/form-data } -form {
    {new_folders_width:integer           {label "Width of new folder tree?"}
                                         {html { size 5 }}}
    {new_folders_depth:integer           {label "Depth of new folder tree?"}
                                         {html { size 5 }}}
    {new_files_width:integer             {label "Files per folder?"}
                                         {html { size 5 }}}
    {upload_file:file                    {label "Data to load into each file"}}
    {indb_p:text(radio)                    {label "File storage type"}
                                         {options {{LOB t} {Filesystem f}}}}
    {user_perms:text(checkbox),multiple,optional
                                         {label "Permissions to assign to creation user"}
                                         {options {{Read read} {Write write} {Admin admin}}}}
    {reg_user_perms:text(checkbox),multiple,optional
                                         {label "Permissions to assign to registered users"}
                                         {options {{Read read} {Write write} {Admin admin}}}}
} -validate {
    {new_folders_width
        { $new_folders_width > 0 }
        "Width must be greater than zero"
    }
    {new_folders_depth
        { $new_folders_depth > 0 }
        "Depth must be greater than zero"
    }
    {new_files_width
        { $new_files_width >= 0 }
        "Files per folder must be greater than or equal to zero"
    }
} -on_submit {

    array set file_storage [site_node::get -url "/file-storage"]

    if {[catch { set root_folder_id [fs::get_root_folder -package_id $file_storage(object_id)] }]} {
        ad_return_error "Could not find root folder" "This is probably because the file-storage package has not been installed and mounted"
        ad_script_abort
    }

    fill_folder_tree -folder_id $root_folder_id -level 1 -max_level \
        $new_folders_depth -width $new_folders_width -files $new_files_width \
        -file_name "[acs_root_dir]/readme.txt" -upload_file $upload_file -indb_p $indb_p \
        -user_perms $user_perms -reg_user_perms $reg_user_perms

    ad_returnredirect "index"
    ad_script_abort

}

set context {"Add files"}

ad_return_template
