ad_page_contract {

    Add an arbitrary number of forums and threads to the system

    @author Don Baccus (dhogaza@pacifier.com)
    @creation-date 2003-01-07

}

set package_id [ad_conn package_id]
set user_id [ad_conn user_id]
permission::require_permission -party_id $user_id -object_id $package_id -privilege admin

ad_form -name add_forums -html { enctype multipart/form-data } -form {
    {new_forum_base_name:text            {label "Base name of forum?"}}
    {new_forums_count:integer            {label "Number of new forums?"}
                                         {html { size 5 }}}
    {new_threads_count:integer           {label "Number of threads per forum?"}
                                         {html { size 5 }}}
    {new_responses_count:integer         {label "Number of responses per thread?"}
                                         {html { size 5 }}}
} -validate {
    {new_forums_count
        { $new_forums_count > 0 }
        "Number of new forums must be greater than zero"
    }
    {new_threads_count
        { $new_threads_count > 0 }
        "Number of threads per forum must be greater than zero"
    }
    {new_responses_count
        { $new_responses_count >= 0 }
        "Responses per thread must be greater than or equal to zero"
    }
} -on_submit {

    array set file_storage [site_node::get -url "/forums"]

    for {} { $new_forums_count > 0 } { incr new_forums_count -1 } {
        set forum_id [forum::new \
                         -name ${new_forum_base_name}_$new_forums_count \
                         -package_id $file_storage(package_id)]

        for { set i 1} { $i <= $new_threads_count } { incr i } {
            set thread_id [forum::message::new \
                              -forum_id $forum_id \
                              -subject thread_$i \
                              -content content]
            for {set j 1} { $j <= $new_responses_count } { incr j } {
                 set thread_id [forum::message::new \
                                  -forum_id $forum_id \
                                  -parent_id $thread_id \
                                  -subject response_$j \
                                  -content content]
            }
        }
    }

    ad_returnredirect "index"
    ad_script_abort

}
set context {"Add forums"}

ad_return_template
