namespace eval portal {}
namespace eval portal::install {}

ad_proc portal::install::package_install {} {
    Package installation callback proc
} {
    acs_sc::contract::new_from_spec -spec {
        name portal_datasource
        description "Portal datasource interface."
        operations {
            Initialize {
                description {
                    Configure datasource-specific parameters to reasonable values.
                }
                input {
                    element_id:integer
                }
                output {
                }
            }
        }
    }

    apm_source [acs_root_dir]/packages/portal/tcl/datasource-procs.tcl

    # Now define some default layouts.

    set var_list {
        {p_name 1_column}
        {p_description "One column"}
        {p_filename /packages/portal/lib/layouts/simple}
    }
    set layout_id [package_instantiate_object -var_list $var_list portal_layout]

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 1]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list {
        {p_name 2_column}
        {p_description "Two columns"}
        {p_filename /packages/portal/lib/layouts/simple}
    }
    set layout_id [package_instantiate_object -var_list $var_list portal_layout]

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 1]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 2]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list {
        {p_name 3_column}
        {p_description "Three columns"}
        {p_filename /packages/portal/lib/layouts/simple}
    }
    set layout_id [package_instantiate_object -var_list $var_list portal_layout]

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 1]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 2]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 3]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list {
        {p_name left_sidebar}
        {p_description "Left sidebar"}
        {p_filename /packages/portal/lib/layouts/left-sidebar}
    }
    set layout_id [package_instantiate_object -var_list $var_list portal_layout]

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 1]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 2]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list {
        {p_name right_sidebar}
        {p_description "Right sidebar"}
        {p_filename /packages/portal/lib/layouts/right-sidebar}
    }
    set layout_id [package_instantiate_object -var_list $var_list portal_layout]

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 1]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 2]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list {
        {p_name left_right_sidebar}
        {p_description "Left and right sidebars"}
        {p_filename /packages/portal/lib/layouts/left-right-sidebar}
    }
    set layout_id [package_instantiate_object -var_list $var_list portal_layout]

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 1]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 2]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    set var_list [list \
        [list p_layout_id $layout_id] \
        [list p_region 3]]
    package_exec_plsql -var_list $var_list portal_layout add_region

    # And some themes

    set var_list {
        {p_name default}
        {p_description "Default OpenACS Theme"}
        {p_filename /packages/portal/lib/themes/standard-theme}
        {p_resource_dir /resources/portal/themes/standard-theme}
    }
    package_instantiate_object -var_list $var_list portal_theme

    set var_list {
        {p_name blank}
        {p_description "No graphics"}
        {p_filename /packages/portal/lib/themes/nada-theme}
        {p_resource_dir /resources/portal/themes/nada-theme}
    }
    package_instantiate_object -var_list $var_list portal_theme

    set var_list {
        {p_name deco}
        {p_description "An Art Deco Theme"}
        {p_filename /packages/portal/lib/themes/deco-theme}
        {p_resource_dir /resources/portal/themes/deco-theme}
    }
    package_instantiate_object -var_list $var_list portal_theme

    # Now define the portal admin portlet

    portal::datasource::new -spec {
        name        portal_admin_portlet
        description "Portal Administration Portlet"
        pretty_name "Portal Administration"
        application ""
        owner       portal
        template    /packages/portal/lib/portal-admin-portlet
        admin_p     t
        aliases {
            Initialize portal::element::default_initializer
        }
    }

    portal::datasource::new -spec {
        name        subsites_portlet
        description "Subsites Portlet"
        pretty_name "Subsites"
        application ""
        owner       portal
        template    /packages/acs-subsite/lib/subsites
        aliases {
            Initialize portal::element::default_initializer
        }
    }
    
}

ad_proc portal::install::package_uninstall {} {
    Uninstall portal_datasource service contract
} {
    portal::datasource::delete -name portal_admin_portlet
    acs_sc::contract::delete -name portal_datasource
}

