#
#  Copyright (C) 2001, 2002 MIT
#
#  This file is part of dotLRN.
#
#  dotLRN is free software; you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation; either version 2 of the License, or (at your option) any later
#  version.
#
#  dotLRN is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#

# www/element-layout-2.tcl
ad_page_contract {
    Do the actual moving/removing of the elements, or redirect to add.

    @author Don Baccus
    @creation-date 2/26/2004
    @cvs_id $Id: portal-configure-2.tcl,v 1.1 2004/03/14 01:19:10 donb Exp $
} {
    portal_id:integer,notnull
    page_id:integer,optional
    element_id:integer,optional
    theme_id:integer,optional
    layout_id:integer,optional
    name:notnull,optional
    op:notnull
    return_url:notnull
    {anchor ""}
}

permission::require_permission -object_id $portal_id -privilege write

switch $op {

    move_up {
        portal::element::move \
            -element_id $element_id \
            -direction up
    }

    move_down {
        portal::element::move \
            -element_id $element_id \
            -direction down
    }

    move_left {
        portal::element::move \
            -element_id $element_id \
            -direction left
    }

    move_right {
        portal::element::move \
            -element_id $element_id \
            -direction right
    }

    show_here {
        db_transaction {
            portal::element::move_to_page \
                -page_id $page_id \
                -element_id $element_id \
                -region 1
            portal::element::set_state \
                -element_id $element_id \
                -state full
        }
    }

    move_to_page {
        portal::element::move_to_page \
            -page_id $page_id \
            -element_id $element_id \
            -region 1
    }

    hide {
        portal::element::set_state -element_id $element_id -state hidden
    }

    change_theme {
        portal::set_theme_id -portal_id $portal_id -theme_id $theme_id
    }

    add_page {
        portal::page::new -portal_id $portal_id -name $name
    }

    remove_page {
        portal::page::delete -page_id $page_id
    }

    change_layout {
        portal::page::set_layout_id \
            -page_id $page_id \
            -layout_id $layout_id
    }

    rename_page {
        portal::page::set_name -page_id $page_id -name $name
    }

    default {
        ad_return_complaint 1 "\"$op\" is not a valid operator for portal configuration"
    }
}

ad_returnredirect $return_url
